from typing import List, Literal

from llmhub.core.secret_managers.base import BaseSecretManager
from llmhub.core.secret_managers.env_var_secrets import EnvSecretManager

from .anthropic import AnthropicClientAsync
from .gemini import GeminiClientAsync
from .template.response import BaseGenerationModel

CLIENT_FACTORY_ASYNC = {"anthropic": AnthropicClientAsync, "gemini": GeminiClientAsync}


class LLMClientAsync:
    def __init__(
        self,
        providers: List[Literal["gemini", "anthropic"]] = ["gemini"],
        secret_manager: BaseSecretManager = EnvSecretManager(),
    ):
        assert isinstance(providers, list)
        self.clients = {}
        for provider in providers:
            if provider in CLIENT_FACTORY_ASYNC.keys():
                self.clients[provider] = CLIENT_FACTORY_ASYNC[provider](secret_manager)
            else:
                print(provider)
                # TODO Warn user here

    async def create_generation(self, input_model: BaseGenerationModel, provider: str):
        assert provider in self.clients.keys()
        return await self.clients[provider].create_generation(input_model)



import os

from llmhub.core.secret_managers.base import BaseSecretManager


class EnvSecretManager(BaseSecretManager):
    def get_secret(self, name: str) -> str:
        value = os.getenv(name)
        if value is None:
            raise KeyError(f"Environment variable '{name}' not found")
        return value



from llmhub.core.secret_managers.base import BaseSecretManager


class BaseClientAsync:
    def __init__(self, secret_manager: BaseSecretManager, warnings: bool = False):
        self.warnings = warnings

    async def create_response():
        pass

    async def get_response():
        pass

    async def delete_response():
        pass

    async def cancel_response():
        pass

    async def list_response_input():
        pass

    async def create_batch():
        pass

    async def get_batch():
        pass

    async def cancel_batch():
        pass

    async def list_batch():
        pass



from typing import Literal, Optional

try:
    from anthropic import AsyncClient
except ImportError:
    pass

from pydantic import BaseModel, Field

from llmhub.core.llms.template import BaseClientAsync
from llmhub.core.llms.template.response import (
    BaseGenerationModel,
    BaseGenerationResponseModel,
)
from llmhub.core.secret_managers.base import BaseSecretManager


class PDFDocumentPart(BaseModel):
    type: Optional[str] = Field("base64")
    media_type: Optional[str] = Field("application/pdf")
    data: str = Field(...)


class Part(BaseModel):
    """A datatype containing media content.

    Exactly one field within a Part should be set, representing the specific type
    of content being conveyed. Using multiple fields within the same `Part`
    instance is considered invalid.
    """

    type: Literal[
        "document",
        "image",
        "redacted_thinking",
        "server_tool_use",
        "text",
        "thinking",
        "tool_result",
        "tool_use",
        "web_search_tool_result",
    ] = Field(...)

    text: Optional[str] = Field(
        default=None, description="""Optional. Text part (can be code)."""
    )

    source: Optional[PDFDocumentPart] = Field(None)

    @classmethod
    def from_text(cls, *, text: str, **args) -> "Part":
        part = cls(text=text, type="text").dict()
        return {k: v for k, v in part.items() if v is not None}

    @classmethod
    def from_base64(cls, *, file_data: str, **args) -> "Part":
        part = cls(type="document", source=PDFDocumentPart(data=file_data)).dict()
        return {k: v for k, v in part.items() if v is not None}


class AnthropicClientAsync(BaseClientAsync):
    def __init__(self, secret_manager: BaseSecretManager):
        self.client = AsyncClient(
            api_key=secret_manager.get_secret("ANTHROPIC_API_KEY")
        )
        self.PART_TYPE_CACTORY = {
            "text": Part.from_text,
            "input_file": Part.from_base64,
        }

    def format_content(self, messages):
        return [
            {
                "role": content["role"],
                "content": [
                    self.PART_TYPE_CACTORY[part["type"]](**part)
                    for part in content["content"]
                ],
            }
            for content in messages
        ]

    async def create_generation(self, input_model: BaseGenerationModel):
        response = await self.client.messages.create(
            model=input_model.model,
            max_tokens=input_model.max_tokens,
            messages=self.format_content(input_model.messages),
        )

        parsed_response = BaseGenerationResponseModel(
            created_at=int(0),
            instructions=input_model.system,
            model=response.model,
            object="response",
            output=[
                {
                    "content": [
                        {
                            "annotations": [],
                            "text": j.text,
                            "type": "output_text",
                            "logprobs": [],
                        }
                        for j in response.content
                    ],
                    "id": response.id,
                    "role": response.role,
                    "status": "completed",
                    "type": "message",
                }
            ],
            status="completed",
        )
        return parsed_response



import boto3

from llmhub.core.secret_managers.base import BaseSecretManager


class AWSSecretManager(BaseSecretManager):
    def __init__(self, region_name: str = "us-east-1"):
        self.client = boto3.client("secretsmanager", region_name=region_name)

    def get_secret(self, name: str) -> str:
        response = self.client.get_secret_value(SecretId=name)
        return response.get("SecretString", "")



from azure.identity import DefaultAzureCredential
from azure.keyvault.secrets import SecretClient

from llmhub.core.secret_managers.base import BaseSecretManager


class AzureKeyVault(BaseSecretManager):
    def __init__(self, vault_url: str):
        credential = DefaultAzureCredential()
        self.client = SecretClient(vault_url=vault_url, credential=credential)

    def get_secret(self, name: str) -> str:
        secret = self.client.get_secret(name)
        return secret.value



from abc import ABC, abstractmethod
from typing import List

from fastapi import FastAPI, UploadFile


class BaseRAG:
    def __init__(
        self,
        vector_store,
    ):
        pass

    def ingest(self):
        pass

    def query(self):
        pass

    def start_api(self, port, host) -> FastAPI:
        app = FastAPI()

        @app.post("/ingest")
        async def ingest_endpoint(file: UploadFile):
            self.ingest(file)
            return {"status": "success"}

        @app.get("/query")
        async def query_endpoint(q: str):
            return self.query(q)

        @app.get("/logs")
        def logs_endpoint():
            return self.logger.get_logs()

        return app


class BaseChunker(ABC):
    def __init__(self, chunk_size: int = 500, chunk_overlap: int = 50):
        self.chunk_size = chunk_size
        self.chunk_overlap = chunk_overlap

    @abstractmethod
    def chunk(self, text: str) -> List[str]:
        """
        Split input text into chunks.
        Must be implemented by subclasses.
        """
        pass



from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, Field


class BaseGenerationModel(BaseModel):
    model: str = Field(..., description="The name of the model to use")
    messages: Any = Field(...)
    system: Optional[str] = Field("", description="")
    max_tokens: Optional[int] = Field(None)
    response_schema: Optional[Any] = Field(None)


class ErrorObject(BaseModel):
    message: str = Field(..., description="Error message.")
    type: Optional[str] = Field(None, description="Error type.")
    code: Optional[str] = Field(None, description="Error code.")


class IncompleteDetails(BaseModel):
    reason: Optional[str] = Field(
        None, description="Reason why the response is incomplete."
    )


class Reasoning(BaseModel):
    mode: Optional[str] = Field(
        None, description="Configuration option for o-series reasoning models."
    )


class PromptReference(BaseModel):
    template_id: Optional[str] = Field(
        None, description="Reference ID for a prompt template."
    )
    variables: Optional[Dict[str, Any]] = Field(
        None, description="Variables used in the template."
    )


class BaseGenerationResponseModel(BaseModel):
    background: Optional[bool] = Field(
        None,
        description="Whether to run the model response in the background.",
        example=False,
    )
    created_at: int = Field(
        ...,
        description="Unix timestamp (in seconds) of response creation.",
        example=1722230400,
    )
    error: Optional[ErrorObject] = Field(
        None, description="An error object if the model failed."
    )
    id: Optional[str] = Field(
        str, description="Unique identifier for this Response.", example="resp_abc123"
    )
    incomplete_details: Optional[IncompleteDetails] = Field(
        None, description="Why the response was incomplete."
    )
    instructions: Union[str, List[str]] = Field(
        ...,
        description="System/developer instructions for the model.",
        example="You are a helpful assistant.",
    )
    max_output_tokens: Optional[int] = Field(
        None, description="Maximum tokens allowed in output.", example=500
    )
    max_tool_calls: Optional[int] = Field(
        None, description="Maximum number of tool calls allowed.", example=2
    )
    metadata: Optional[Dict[str, str]] = Field(
        None,
        description="Key-value metadata (up to 16 pairs).",
        example={"session_id": "abc123"},
    )
    model: str = Field(..., description="ID of the model used.", example="gpt-4o")
    object: str = Field(
        ..., description="Object type, always 'response'.", example="response"
    )
    output: List[Any] = Field(
        ..., description="Generated content items from the model."
    )
    output_text: Optional[str] = Field(
        None,
        description="Aggregated text output from all content items.",
        example="Hello! How can I help you?",
    )
    parallel_tool_calls: Optional[bool] = Field(
        None, description="Allow parallel tool calls.", example=True
    )
    previous_response_id: Optional[str] = Field(
        None, description="ID of the previous response.", example="resp_prev456"
    )
    prompt: Optional[PromptReference] = Field(
        None, description="Reference to a prompt template and its variables."
    )
    prompt_cache_key: Optional[str] = Field(
        None,
        description="Key used for caching similar prompts.",
        example="cache_key_xyz789",
    )
    reasoning: Optional[Reasoning] = Field(
        None, description="Reasoning config for o-series models."
    )
    safety_identifier: Optional[str] = Field(
        None,
        description="Stable identifier to detect policy violations.",
        example="user_hash_123",
    )
    service_tier: Optional[str] = Field(
        None, description="Processing tier used.", example="default"
    )
    status: str = Field(..., description="Generation status.", example="completed")
    temperature: Optional[float] = Field(
        None, description="Sampling temperature (0 to 2).", example=0.7
    )
    text: Optional[Dict[str, Any]] = Field(
        None, description="Text response configuration.", example={"format": "plain"}
    )
    tool_choice: Optional[Union[str, Dict[str, Any]]] = Field(
        None, description="Tool selection strategy.", example="auto"
    )
    tools: Optional[List[Dict[str, Any]]] = Field(
        None, description="Array of tool definitions."
    )
    top_logprobs: Optional[int] = Field(
        None,
        description="Number of top log probabilities to return per token.",
        example=5,
    )
    top_p: Optional[float] = Field(
        None, description="Top-p (nucleus) sampling probability.", example=0.9
    )
    truncation: Optional[str] = Field(
        None, description="Truncation strategy.", example="auto"
    )
    usage: Optional[Dict[str, Any]] = Field(
        None,
        description="Token usage breakdown.",
        example={"input_tokens": 10, "output_tokens": 20},
    )



import base64
from typing import Any, Optional

try:
    import magic
    from google.genai import Client, types
except ImportError:
    pass

from pydantic import BaseModel, Field

from llmhub.core.llms.template import BaseClientAsync
from llmhub.core.llms.template.response import (
    BaseGenerationModel,
    BaseGenerationResponseModel,
)
from llmhub.core.secret_managers.base import BaseSecretManager


def get_mimetype(file_bytes: bytes) -> str:
    return magic.from_buffer(file_bytes[:2048], mime=True)


class Part(BaseModel):
    """A datatype containing media content.

    Exactly one field within a Part should be set, representing the specific type
    of content being conveyed. Using multiple fields within the same `Part`
    instance is considered invalid.
    """

    video_metadata: Optional[types.VideoMetadata] = Field(
        default=None, description="""Metadata for a given video."""
    )
    thought: Optional[bool] = Field(
        default=None,
        description="""Indicates if the part is thought from the model.""",
    )
    inline_data: Optional[types.Blob] = Field(
        default=None, description="""Optional. Inlined bytes data."""
    )
    file_data: Optional[types.FileData] = Field(
        default=None, description="""Optional. URI based data."""
    )
    thought_signature: Optional[bytes] = Field(
        default=None,
        description="""An opaque signature for the thought so it can be reused in subsequent requests.""",
    )
    code_execution_result: Optional[types.CodeExecutionResult] = Field(
        default=None,
        description="""Optional. Result of executing the [ExecutableCode].""",
    )
    executable_code: Optional[types.ExecutableCode] = Field(
        default=None,
        description="""Optional. Code generated by the model that is meant to be executed.""",
    )
    function_call: Optional[types.FunctionCall] = Field(
        default=None,
        description="""Optional. A predicted [FunctionCall] returned from the model that contains a string representing the [FunctionDeclaration.name] with the parameters and their values.""",
    )
    function_response: Optional[types.FunctionResponse] = Field(
        default=None,
        description="""Optional. The result output of a [FunctionCall] that contains a string representing the [FunctionDeclaration.name] and a structured JSON object containing any output from the function call. It is used as context to the model.""",
    )
    text: Optional[str] = Field(
        default=None, description="""Optional. Text part (can be code)."""
    )

    @classmethod
    def from_uri(cls, *, file_uri: str, mime_type: Optional[str] = None) -> "Part":
        """Creates a Part from a file uri.

        Args:
          file_uri (str): The uri of the file
          mime_type (str): mime_type: The MIME type of the image. If not provided,
            the MIME type will be automatically determined.
        """
        if mime_type is None:
            import mimetypes

            mime_type, _ = mimetypes.guess_type(file_uri)
            if not mime_type:
                raise ValueError(f"Failed to determine mime type for file: {file_uri}")
        file_data = types.FileData(file_uri=file_uri, mime_type=mime_type)
        return cls(file_data=file_data)

    @classmethod
    def from_text(cls, *, text: str, **args) -> "Part":
        return cls(text=text)

    @classmethod
    def from_base64(cls, *, file_data: str, **args) -> "Part":
        file_data = base64.b64decode(file_data)
        inline_data = types.Blob(
            data=file_data,
            mime_type=get_mimetype(file_data),
        )
        return cls(inline_data=inline_data)

    @classmethod
    def from_function_call(cls, *, name: str, args: dict[str, Any]) -> "Part":
        function_call = types.FunctionCall(name=name, args=args)
        return cls(function_call=function_call)

    @classmethod
    def from_function_response(cls, *, name: str, response: dict[str, Any]) -> "Part":
        function_response = types.FunctionResponse(name=name, response=response)
        return cls(function_response=function_response)

    @classmethod
    def from_executable_code(cls, *, code: str, language: types.Language) -> "Part":
        executable_code = types.ExecutableCode(code=code, language=language)
        return cls(executable_code=executable_code)

    @classmethod
    def from_code_execution_result(
        cls, *, outcome: types.Outcome, output: str
    ) -> "Part":
        code_execution_result = types.CodeExecutionResult(
            outcome=outcome, output=output
        )
        return cls(code_execution_result=code_execution_result)


class GeminiClientAsync(BaseClientAsync):
    def __init__(self, secret_manager: BaseSecretManager):
        self.client = Client(api_key=secret_manager.get_secret("GEMINI_API_KEY"))
        self.aio_client = self.client.aio
        self.PART_TYPE_CACTORY = {
            "text": Part.from_text,
            "input_file": Part.from_base64,
        }

    def format_content(self, contents):
        return [
            types.Content(
                role=content["role"],
                parts=[
                    self.PART_TYPE_CACTORY[part["type"]](**part)
                    if not isinstance(part, str)
                    else Part(text=part)
                    for part in content["content"]
                ],
            )
            for content in contents
        ]

    async def create_generation(self, input_model: BaseGenerationModel):
        if input_model.response_schema:
            schema = input_model.response_schema.schema()
            schema["propertyOrdering"] = list(schema["properties"].keys())
        response = await self.aio_client.models.generate_content(
            model=input_model.model,
            contents=self.format_content(input_model.messages),
            config=types.GenerateContentConfig(
                system_instruction=input_model.system,
                response_mime_type="application/json"
                if input_model.response_schema
                else None,
                response_schema=schema if input_model.response_schema else None,
            ),
        )

        parsed_response = BaseGenerationResponseModel(
            created_at=int(response.create_time or 0),
            instructions=input_model.system,
            model=response.model_version,
            object="response",
            output=[
                {
                    "content": [
                        {
                            "annotations": [],
                            "text": j.text,
                            "type": "output_text",
                            "logprobs": [],
                        }
                        for j in i.content.parts
                    ],
                    "id": i.index,
                    "role": i.content.role,
                    "status": "completed",
                    "type": "message",
                }
                for i in response.candidates
            ],
            status="completed",
        )
        return parsed_response
